const opentype = require('opentype.js');

opentype.load(__dirname + '/SourceHanSansSC-Regular.otf', (error, font) => {
    if (error) {
        throw error;
    }

    let gid = 0;
    const set = new Set();
    const glyphs = font.stringToGlyphs('汉体  ').filter(glyph => {
        if (set.has(glyph.index)) {
            return false;
        }
        
        if (typeof glyph.name !== 'string') {
            // 修复一些字体缺失的数据
            glyph.name = `gid${gid ++}`;
        }

        set.add(glyph.index);
        return true;
    });

    const familyName = font.getEnglishName('fontFamily');
    const styleName = font.getEnglishName('fontSubfamily');

    const otf = new opentype.Font({
        familyName,
        styleName,
        unitsPerEm: font.unitsPerEm,
        ascender: font.ascender,
        descender: font.descender,
        glyphs: glyphs,
        //manufacturer: 'font-spider',
        //manufacturerURL: 'http://font-spider.org',
        description: 'created by font-spider@v1.0.0'
    });
    var arrayBuffer = otf.toArrayBuffer();
    function arrayBufferToNodeBuffer(ab) {
        var buffer = new Buffer(ab.byteLength);
        var view = new Uint8Array(ab);
        for (var i = 0; i < buffer.length; ++i) {
            buffer[i] = view[i];
        }

        return buffer;
    }
    var buffer = arrayBufferToNodeBuffer(arrayBuffer);
    otf.download(__dirname + '/min.otf');

});